/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#include  <dbStaticLib.h>
#include <stdio.h>
#include <string.h>
#include  <stdlib.h>
#include  <stddef.h>

#include <tcl.h>
#include <tk.h>
 
#include  "epicsVersion.h"

#if  EPICS_REVISION && EPICS_REVISION <= 13
#define dbFindRecordType    dbFindRecdes
#define dbGetNRecordTypes   dbGetNRecdes
#define dbNextRecordType    dbNextRecdes
#define dbFirstField        dbFirstFielddes
#define dbGetRecordTypeName dbGetRecdesName
#define dbFirstRecordType   dbFirstRecdes
#define dbNextField         dbNextFielddes
#define dbGetMenuChoices    dbGetChoices
#endif

#include  <APSlib.h>
#include  <dct.h>     /* this is used for  dct.c production code  */


DBENTRY *pdbentry ;      /* allocate             DBENTRY structure  */
DBBASE  *pdbbase = 0 ;   /* allocate & initialize DBBASE structure  */ 
dbMenu  *pdbmenu = 0 ;   /* allocate & initialize dbMenu structure  */ 
char str[1000];

/* Number 1 ....................................................... */
int  DCTdbAllocEntry (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
{
    if (pdbbase && !pdbentry)  {        
	pdbentry = dbAllocEntry (pdbbase) ;
	return  TCL_OK ;
    }
}

/* Number 1a ....................................................... */
int  DCTdbFreeEntry (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] ) 
{
    if (pdbentry)  {        
	dbFreeEntry (pdbentry) ;
	pdbentry = 0;
	return  TCL_OK ;
    }
}

/* Number 2 ....................................................... */
int  DCTdbFreeBase (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
{
    if (pdbbase)  {        
	dbFreeBase (pdbbase) ;
	pdbbase = 0;
	return  TCL_OK ;
    }
} 

/* Number 27 ....................................................... */
int  DCTdbAllocBase (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
{
    if (!pdbbase)  {        
	pdbbase = dbAllocBase () ;
	return  TCL_OK ;
    }
} 

/* Number 3 ....................................................... */
int  DCTdbGetNRecordTypes (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
{  int num ;   
    if (pdbentry)  {        
	num   = dbGetNRecordTypes (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }
 
/* Number 4 ....................................................... */
 int  DCTdbFirstRecordType (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFirstRecordType (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbFirstRecordType: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 5 ....................................................... */
 int  DCTdbNextRecordType (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbNextRecordType (pdbentry) ;
/*	if (lnum) printf("*** ERROR in DCTdbNextRecordType: non-zero status \n");
*/

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 6 ....................................................... */
 int  DCTdbFindRecordType (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFindRecordType (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbFindRecordType: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 7 ....................................................... */
 int  DCTdbGetRecordTypeName (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char * name ;   
    if (pdbentry)  {        
	name   = dbGetRecordTypeName (pdbentry) ;
	sprintf (interp->result, "%-25s"   , name   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 8 ....................................................... */
 int  DCTdbGetNRecords (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetNRecords (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 9 ....................................................... */
 int  DCTdbFirstRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFirstRecord (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbFirstRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 10 ....................................................... */
 int  DCTdbNextRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbNextRecord (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbNextRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 11 ....................................................... */
 int  DCTdbFindRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFindRecord (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbFindRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 12 ....................................................... */
 int  DCTdbGetRecordName (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char * name ;   
    if (pdbentry)  {        
	name   = dbGetRecordName (pdbentry) ;
/*	sprintf (interp->result, "%-28s"   , name   ) ;  */
	strcpy  (interp->result, name) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 13 ....................................................... */
 int  DCTdbGetNFields(ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetNFields (pdbentry,TRUE) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }
  
/* Number 14 ....................................................... */
 int  DCTdbFirstField (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFirstField (pdbentry, TRUE ) ;
	if (lnum) printf("*** ERROR in DCTdbFirstField: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }
  
/* Number 15 ....................................................... */
 int  DCTdbNextField (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbNextField (pdbentry, TRUE ) ;
/*
if (lnum) printf("*** ERROR in DCTdbNextField: non-zero status \n");
*/
	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 16 ....................................................... */
 int  DCTdbFindField (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
/*  printf("DCTdbFindField field |%s|\n",argv[1]);  */
	lnum   = dbFindField (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbFindField: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }
      
/* Number 17 ....................................................... */
 int  DCTdbGetFieldName (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char *name ;   
    if (pdbentry)  {        
	name   = dbGetFieldName (pdbentry) ;
	if(name) {
	    strcpy(interp->result,name);
	    return TCL_OK;
	} else {
	    return  TCL_ERROR ;
	}
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 18 ....................................................... */
 int  DCTdbGetFieldType (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetFieldType (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 19 ....................................................... */
 int  DCTdbGetPrompt (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char *name ;   
    if (pdbentry)  {        
	name   = dbGetPrompt (pdbentry) ;
	sprintf (interp->result, "%-22s"   , name   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 20 ....................................................... */
 int  DCTdbGetPromptGroup (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetPromptGroup (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

 
/* Number 21 ....................................................... */
int  DCTdbGetRange (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char *name ;   
    if (pdbentry)  {        
	name   = dbGetRange (pdbentry) ;
	sprintf (interp->result, "%-8s"   , name   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

 /* Number 22 ....................................................... */
 int  DCTdbCreateRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbCreateRecord (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbCreateRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 23 ....................................................... */
 int  DCTdbDeleteRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbDeleteRecord (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbDeleteRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 24 ....................................................... */
 int  DCTdbRenameRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbRenameRecord (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbRenameRecord: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 25 ....................................................... */
 int  DCTdbReadDatabase (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   

    if (pdbbase)  {        
/*  printf ("25.0  ... call dbDumpRecord\n"); */
/*       dbDumpRecord(pdbbase,0,0);           */

/* printf ("25.1 argc=%d \n",argc);  */
/* printf ("25.2 arg1=%s \n",argv[1]); */
/* printf ("25.3 arg2=%s \n",argv[2]); */
	lnum   = dbReadDatabase (&pdbbase, argv[1], argv[2], 0) ;
/*	printf ("25.4 dbReadDatabase is done  ... \n");  */
	if (lnum)  printf("25.5 dbReadDatabase  attempt to read failed\n");
	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 26 ....................................................... */
 int  DCTdbWriteRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    int intargv2;

    sscanf (argv[2], "%d", &intargv2) ;

/*  printf ("26.1 argv1=%s \n",argv[1]); */ 
/*  printf ("26.2 argv2=%d \n",intargv2); */ 

    if (pdbbase)  {        
/*  printf ("26.3 before dbWriteRecord call \n"); */ 
	lnum   = dbWriteRecord (pdbbase, argv[1], 0  , intargv2 ) ;
	if (lnum) printf("*** ERROR in DCTdbWriteRecord: non-zero status \n");

/*  printf ("26.4 after dbWriteRecord call OK"); */ 
	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 28 ....................................................... */
 int  DCTdbGetString (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char * string ;   
    if (pdbentry)  {        
	string   = dbGetString (pdbentry) ;
	if (string)
	   strcpy(interp->result,string);
        else
	   interp->result[0]=0;

	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 29 ....................................................... */
 int  DCTdbGetMenuIndex (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int index ;   
    if (pdbentry)  {        
	index   = dbGetMenuIndex (pdbentry) ;
	sprintf (interp->result, "%d"   , index   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 30 ..................................................... */

 int  DCTdbGetMenuChoices (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char **choices ;   
    char *next;
    int  numc,j ;
    if (pdbentry)  {        
	numc = dbGetNMenuChoices(pdbentry);
/* printf ("\n GetChoices  numc=%d " ,numc);   */
	if (numc > 0)  {
	     choices  = dbGetMenuChoices(pdbentry) ;
	     next = str;
	     for (j=0;  j<numc;  j++)  {
/* printf ( "\"%s\" " , choices[j] ) ;         */
		 sprintf (next, "\"%s\" " , choices[j] ) ;
		 next = next + strlen(choices[j])+ 3;
		 }
/* printf ( "DONE\n" ) ;              */
	     interp->result = str;
	     return  TCL_OK ;
	} else  {
	     sprintf (interp->result, "0"  ) ;   
	     return  TCL_OK ;
	}

     }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 54 (like 30) .............................................  */

 int  DCTdbGetFormPrompt (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char **choices ;   
    char *next;
    int  numc,j ;
    if (pdbentry)  {        
	numc = dbAllocForm(pdbentry);
/* printf("DCTdbGetFormPrompt numc %d\n",numc); */
	if (numc > 0)  {
	     choices  = dbGetFormPrompt(pdbentry) ;
/* printf("choices %p\n",choices);  */
	     next = str;
	     for (j=0;  j<numc;  j++)  {
/* printf("DCTdbGetFormPrompt choices[j] %s\n",choices[j]);  */
		 sprintf (next, "\"%s\" " , choices[j] ) ;
		 next = next + strlen(choices[j])+ 3;
		 }
	     dbFreeForm(pdbentry);
	     interp->result = str;
	     return  TCL_OK ;
	} else  {
	     sprintf (interp->result, ""  ) ;   
	     dbFreeForm(pdbentry);
	     return  TCL_ERROR ;
	}

     }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

 /* Number 55 (like 30 andd 54) ................................... */

 int  DCTdbGetFormValue (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char **choices ;   
    char *next;
    int  numc,j ;
    if (pdbentry)  {        
	numc = dbAllocForm(pdbentry);
/* printf("DCTdbGetFormValue numc %d\n",numc); */
	if (numc > 0)  {
	     choices  = dbGetFormValue(pdbentry) ;
	     next = str;
	     for (j=0;  j<numc;  j++)  {
/* printf("DCTdbGetFormValue choices[j] %s\n",choices[j]); */
		 sprintf (next, "\"%s\" " , choices[j] ) ;
		 next = next + strlen(choices[j])+ 3;
		 }
	     dbFreeForm(pdbentry);  
	     interp->result = str;
	     return  TCL_OK ;
	} else  {
	     sprintf (interp->result, ""  ) ;   
	     dbFreeForm(pdbentry);
	     return  TCL_ERROR ;
	}

     }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }
/* Number 31 ....................................................... */
 int  DCTdbCopyRecord (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   

    if (pdbentry)  {        
	lnum   = dbCopyRecord (pdbentry, argv[1], 0 ) ; 

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 32 ....................................................... */
 int  DCTdbWriteDefinitions (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbbase)  {        

	  return  TCL_ERROR ;
	}
 }

/* Number 33 .................................................... */
 int  DCTdbPutString (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbPutString (pdbentry, argv[1] ) ;
	if (lnum) printf("*** ERROR in DCTdbPutString: non-zero status \n");

/* printf ("... 33.1  C.  dbPutString has finished \n");  */

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 34 ....................................................... */
int  DCTdbVerify (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char * name ;   

    if (pdbentry)  {        
	name   = dbVerify (pdbentry,argv[1]) ;
	if(name)
	    strcpy(interp->result,name);
	else
	    interp->result[0]=0;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 35 ....................................................... */
 int  DCTdbIsDefaultValue (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbIsDefaultValue (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }  

/* Number 36 ....................................................... 

int  DCTdbFindMenu (ClientData  clientData, Tcl_Interp *interp,
*/


/* Number 50 ....................................................... */
 int  DCTdbPutMenuIndex (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    int  intargv1;
    if (pdbentry)  {        
	sscanf (argv[1], "%d", intargv1);
	lnum   = dbPutMenuIndex (pdbentry, intargv1 ) ;
	if (lnum) printf("*** ERROR in DCTdbPutMenuIndex: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 51 ....................................................... */
 int  DCTdbGetNMenuChoices (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetNMenuChoices (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 52 ....................................................... */
 int  DCTdbAllocForm (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbAllocForm (pdbentry) ;
/*  printf ("C.AllocForm num= %d\n "   , num   ) ;  */
	sprintf (interp->result, "%d"   , num   ) ;
	dbFreeForm (pdbentry) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 53 ....................................................... */
 int  DCTdbFreeForm (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbFreeForm (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbFreeForm: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }  



/* Number 58 ....................................................... */
 int  DCTdbGetNLinks (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetNLinks (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 59 ....................................................... */
 int  DCTdbGetLinkField (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    int  intargv1; 
    if (pdbentry)  {        
	sscanf (argv[1], "%d", &intargv1);
	lnum   = dbGetLinkField (pdbentry, intargv1 ) ;
	if (lnum) printf("*** ERROR in DCTdbGetLinkField: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 60 ....................................................... */
 int  DCTdbGetLinkType (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
    if (pdbentry)  {        
	num   = dbGetLinkType (pdbentry) ;
	sprintf (interp->result, "%d"   , num   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 61 ....................................................... */
 int  DCTdbCvtLinkToConstant (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbCvtLinkToConstant (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbCvtLinkToConst: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }

/* Number 62 ....................................................... */
 int  DCTdbCvtLinkToPvlink (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  long lnum ;   
    if (pdbentry)  {        
	lnum   = dbCvtLinkToPvlink (pdbentry) ;
	if (lnum) printf("*** ERROR in DCTdbCvtLinkToPvlink: non-zero status \n");

	sprintf (interp->result, "%ld"   , lnum   ) ;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 63 ....................................................... 
 int  DCTdbPutPvlink (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
*/   


/* Number 64 ....................................................... 
 int  DCTdbGetPvlink (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
*/   


/* Number 65 ....................................................... */
 int  DCTdbDelAllRecIns (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
{ int i, j, no1;
  int  numRecTypes;
  int  numRecords;

  long  lnum1;
  long  lnum2;
  long  lnum3;
  long  lnum4;

  if (pdbentry)  {
     numRecTypes = dbGetNRecordTypes(pdbentry) ;
     lnum1 = dbFirstRecordType(pdbentry) ;

     for (j=0;  j<numRecTypes;  j++)  {
	numRecords = dbGetNRecords(pdbentry);

	if (numRecords  == 0)  {
	  lnum2 = dbNextRecordType(pdbentry);
          continue;
        }
 /* printf ("C ... num recs to be deleted=%d \n",numRecords);  */

	for (i=0;  i<numRecords;  i++)  {
	  no1 = dbFirstRecord(pdbentry);
	  lnum3 = dbDeleteRecord(pdbentry);
        }

	lnum4 = dbNextRecordType(pdbentry);
     }

     return  TCL_OK ;
   }
    else {
       sprintf(interp->result, "");
       return  TCL_ERROR ;
   }

 }

/* Number 66 ........................................................*/
 int  DCTdbPutFormLine (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char **value;
    char **prompt;
    char **error;
    int  n ;
    int  i ;
    int  index ;
    long lnumPut;

    n = dbAllocForm(pdbentry) ;

    prompt = dbGetFormPrompt(pdbentry);

    value = dbGetFormValue(pdbentry);

/*    index = 1 ;       */
/* printf ("... DCTdbPutFormLine: argv[1]= %s \n", argv[1] ) ; */

/*    index = argv[1] ;  */

    sscanf(argv[1], "%d" , &index ) ; 

/* printf ("... DCTdbPutFormLine: index = %d \n", index) ;  */

    strcpy (value[index], argv[2] ) ;

/* printf ("... DCTdbPutFormLine: no. of form lines =%d \n ", n) ; */

    for (i=0;  i<n;  i++)  {
      /* printf("%s (%s) : \n", prompt[i], value[i] );  */
    }

/* printf ("... DCTPutFormLine: just b4 dbVerifyForm  call \n") ; */
    error = dbVerifyForm(pdbentry,value) ;
    if (error) {
          for(i=0;  i<n;  i++)  {
	  /* printf( "Error: %s (%s) %s\n", prompt[i], value[i],error[i]); */
	  }
	  /* printf("... all form entry errors processed \n") ; */
    } else { /* printf("... DCTPutFormLine: no form entry errors \n") ; */
	    /* printf("... DCTPutFormLine: just b4 dbPutForm call \n"); */
	    lnumPut = dbPutForm(pdbentry,value) ;
	    if (lnumPut) printf("*** ERROR in dbPutForm: non-zero status");
    }

/* dbFreeForm(pdbentry);  */

/* printf ("... leaving DCTdbPutFormLine ...  \n ") ; */

    return TCL_OK ;
 }



/* Number 67 ....................................................... */
 int  DCTdbGetRelatedField (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  char *name ;   

    if (pdbentry)  {        

	name   = dbGetRelatedField(pdbentry) ;
        if (name) 
	   strcpy(interp->result,name);
        else
	   interp->result[0]=0;
	return  TCL_OK ;
    }
	else  {
	  sprintf (interp->result, ""  ) ;   
	  return  TCL_ERROR ;
	}
 }


/* Number 68 ....................................................... */
 int  DCTdbGetST (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_STRING ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetIN (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_INTEGER ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetRL (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_REAL ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetME (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_MENU ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetMF (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_MENUFORM ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetNL (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_INLINK ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetOL (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_OUTLINK ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 68 ....................................................... */
 int  DCTdbGetFL (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_FWDLINK ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 69a ....................................................... */
 int  DCTdbGetLkPV (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_LINK_PV ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 69b ....................................................... */
 int  DCTdbGetLkC (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_LINK_CONSTANT ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }

/* Number 69c ....................................................... */
 int  DCTdbGetLkF (ClientData  clientData, Tcl_Interp *interp,
			int argc,  const char *argv[] )
 {  int num ;   
	num = DCT_LINK_FORM ;
	sprintf (interp->result, "%d", num );
	return  TCL_OK ;
 }




void createTclCommands(Tcl_Interp *interp) {

    Tcl_CreateCommand (interp, "DCTdbAllocEntry",    DCTdbAllocEntry,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFreeEntry",     DCTdbFreeEntry,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbAllocBase",    DCTdbAllocBase,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFreeBase",    DCTdbFreeBase,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNRecordTypes",  DCTdbGetNRecordTypes,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFirstRecordType", DCTdbFirstRecordType,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbNextRecordType",  DCTdbNextRecordType,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFindRecordType",  DCTdbFindRecordType,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetRecordTypeName", DCTdbGetRecordTypeName,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNRecords", DCTdbGetNRecords,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFirstRecord", DCTdbFirstRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbNextRecord",  DCTdbNextRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFindRecord",  DCTdbFindRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetRecordName", DCTdbGetRecordName,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNFields",  DCTdbGetNFields,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFirstField", DCTdbFirstField,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbNextField", DCTdbNextField,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFindField",   DCTdbFindField,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetFieldName", DCTdbGetFieldName,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetFieldType", DCTdbGetFieldType,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetPrompt",   DCTdbGetPrompt,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand(interp,"DCTdbGetPromptGroup", DCTdbGetPromptGroup,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetString",   DCTdbGetString,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetRange",    DCTdbGetRange,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbCreateRecord", DCTdbCreateRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbCopyRecord", DCTdbCopyRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbDeleteRecord", DCTdbDeleteRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbRenameRecord", DCTdbRenameRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbReadDatabase", DCTdbReadDatabase,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbAllocBase", DCTdbAllocBase,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbWriteRecord", DCTdbWriteRecord,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbWriteDefinitions", DCTdbWriteDefinitions,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbPutString", DCTdbPutString,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbVerify",  DCTdbVerify,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbIsDefaultValue", DCTdbIsDefaultValue,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbPutMenuIndex",  DCTdbPutMenuIndex,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNMenuChoices", DCTdbGetNMenuChoices,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbAllocForm",  DCTdbAllocForm,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbFreeForm", DCTdbFreeForm,  
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetFormPrompt", DCTdbGetFormPrompt,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetFormValue", DCTdbGetFormValue,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNLinks", DCTdbGetNLinks,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetLinkField", DCTdbGetLinkField,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetLinkType", DCTdbGetLinkType,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand(interp,"DCTdbCvtLinkToConstant",DCTdbCvtLinkToConstant,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbCvtLinkToPvlink", DCTdbCvtLinkToPvlink,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbGetMenuChoices", DCTdbGetMenuChoices, 
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbGetMenuIndex",  DCTdbGetMenuIndex,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbPutFormLine",  DCTdbPutFormLine,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbDelAllRecIns", DCTdbDelAllRecIns,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
  
    Tcl_CreateCommand (interp, "DCTdbGetRelatedField", DCTdbGetRelatedField,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbGetST", DCTdbGetST,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetIN", DCTdbGetIN,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetRL", DCTdbGetRL,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetME", DCTdbGetME,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetMF", DCTdbGetMF,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetNL", DCTdbGetNL,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetOL", DCTdbGetOL,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetFL", DCTdbGetFL,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;

    Tcl_CreateCommand (interp, "DCTdbGetLkPV", DCTdbGetLkPV,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetLkC", DCTdbGetLkC,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
    Tcl_CreateCommand (interp, "DCTdbGetLkF", DCTdbGetLkF,
	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL  ) ;
}


/* the following stmts for dct.c production code only                 */

int  main(int argc, char **argv)
{
    int   i;
    int   num;
    int   nof;
    int   status = 0 ;
    Tk_Window mainWindow;
	Tcl_Interp *interp;
	static char *display = NULL;

    pdbbase = dbAllocBase () ;

    for (i=1; i<argc;  i++) {
	status = dbReadDatabase(&pdbbase, argv[i],  0, 0) ;
	if (status) {
	    printf ("%s *** read for  .dbd  Failed\n", argv[1] ) ;
	    return(-1);
        }
    }

	interp = Tcl_CreateInterp();

/*
	 mainWindow = Tk_CreateMainWindow(interp, display, argv[0], "Tk"); 
	if (mainWindow == NULL) {
		fprintf(stderr, "%s\n", interp->result);
		return(-1);
	}
*/

	if (Tcl_Init(interp) == TCL_ERROR) {
		fprintf(stderr, "Tcl_Init failed: %s\n", interp->result);
	}

	if (Tk_Init(interp) == TCL_ERROR) {
		fprintf(stderr, "Tk_Init failed: %s\n", interp->result);
	}

    createTclCommands(interp);

    pdbentry = dbAllocEntry(pdbbase);

	Tcl_Eval (interp, APSlib );

	Tcl_Eval (interp, dct  ) ;

	Tk_MainLoop();

	return(0);
}



